/*
******************************************************************************
*
*      File             : tfo_main.c
*      Version          : 2.0.0
*      Date             : 27 February 2001
*      Author           : Jochen Junkawitsch
*      Description      : source file for TFO decision algorithm
*
*
******************************************************************************
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "tfo_decision.h"
#include <iostream.h>                     // 4.3.0 for debugging




/* ------------------------------------------------------------------- */


/* convert a string (example "-X--XX-X") to a CodecSet */
CodecSet s2cs(char* s)
{
  int k;
  CodecSet m, cs = 0;
  
  if (strlen(s) != 8)
	{fprintf(stderr,"ERROR in input: codec set not 8 characters long\n");exit(0);} //4.3.0 error text
  
  for (k=0, m=mode_122; k<8; k++, m >>= 1)
    if ( (s[k] == 'X') || (s[k] == 'x') )   // 4.3.0 also small x allowed
      cs |= m;
  
  return cs;
}


/* read input Data from stdin */
int getInputData(Data* data)
{
  char buffer[255];

  char lscs[255], dscs[255];
  char lacs[255], dacs[255];
  int lmacs, dmacs;
  char lType[255], dType[255];
  char lOM, dOM;
  
  data->cacs = data->oacs = data->cscs = data->lmacs = 0;  // 4.3.0 lmacs initialized

	// 4.3.0 for easier input
	printf("----------------------------------------------------------------------\n\n"); 
	printf("This program checks if TFO is possible and, if on AMR, on which CACS. \n"); 
	printf("Expected input: \n"); 
	printf("lSCS lACS lMACS lType lOM dSCS dACS dMACS dType dOM   (l: local, d: distant)\n");
	printf("For all codec sets highest mode comes first; x: mode present, -: not present\n"); 
	printf("Examples:\n");
	printf("-------- -------- 0 GSM_EFR y -------- -------- 0 GSM_EFR n \n"); 
	printf("xxxxxxxx x-x-x-x- 4 FR_AMR y --xxxxxx ---x-x-x 3 HR_AMR n \n"); 
	printf("xxxxxxxx -x-x-x-x 4 UMTS_AMR y xxxxxxxx --x-x-x- 3 UMTS_AMR2 n \n"); 
	printf("Your input, please: [to exit push the Return button two times]\n"); 

  if (!fgets(buffer, 255, stdin))
	  return 0;

  if (sscanf(buffer, "%s %s %d %s %c %s %s %d %s %c",
			 lscs, lacs, &lmacs, lType, &lOM,
			 dscs, dacs, &dmacs, dType, &dOM) != 10)
  {fprintf(stderr,"ERROR in input: not 10 parameters\n");exit(0);}  //4.3.0 error text
  

  data->lscs  = s2cs(lscs);
  data->lacs  = s2cs(lacs);

  
  if (!strcmp(lType, "FR_AMR"))
	  data->lType = FR_AMR;
  else if (!strcmp(lType, "HR_AMR"))
	  data->lType = HR_AMR;
  else if (!strcmp(lType, "UMTS_AMR"))
	  data->lType = UMTS_AMR;
  else if (!strcmp(lType, "UMTS_AMR_2"))
	  data->lType = UMTS_AMR_2;
  else if (!strcmp(lType, "GSM_EFR"))
	  data->lType = GSM_EFR;
  else if (!strcmp(lType, "GSM_FR"))
	  data->lType = GSM_FR;
  else if (!strcmp(lType, "GSM_HR"))
	  data->lType = GSM_HR;
  else
  {fprintf(stderr,"ERROR in input: unknown local codec type\n");exit(0);}  //4.3.0 error text
  
   if (                                  // 4.3.0 condition introduced, moved downwards
     data->lType == FR_AMR     ||        // after point where lType is read from input
     data->lType == HR_AMR     ||
     data->lType == UMTS_AMR   || 
     data->lType == UMTS_AMR_2
     )
	 {
     if (lmacs > 0 && lmacs <= 8)
    	  data->lmacs = lmacs;
     else
		{fprintf(stderr,"ERROR in input: lmacs not between 1 and 8\n");exit(0);}  //4.3.0 error text
	 }
   

  if (lOM == 'y')
	  data->lOM = 1;
  else if (lOM == 'n')
	  data->lOM = 0;
  else
  {fprintf(stderr,"ERROR in input: lOM not 'y' or 'n'\n");exit(0);}  //4.3.0 error text

  data->dscs  = s2cs(dscs);
  data->dacs  = s2cs(dacs);


  if (!strcmp(dType, "FR_AMR"))
	  data->dType = FR_AMR;
  else if (!strcmp(dType, "HR_AMR"))
	  data->dType = HR_AMR;
  else if (!strcmp(dType, "UMTS_AMR"))
	  data->dType = UMTS_AMR;
  else if (!strcmp(dType, "UMTS_AMR_2"))
	  data->dType = UMTS_AMR_2;
  else if (!strcmp(dType, "GSM_EFR"))
	  data->dType = GSM_EFR;
  else if (!strcmp(dType, "GSM_FR"))
	  data->dType = GSM_FR;
  else if (!strcmp(dType, "GSM_HR"))
	  data->dType = GSM_HR;
  else
  {fprintf(stderr,"ERROR in input: unknown dType\n");exit(0);}  //4.3.0 error text

    if (                                // 4.3.0 condition introduced, moved downwards
       data->dType == FR_AMR     ||     // after point where dType is read from input
       data->dType == HR_AMR     ||
       data->dType == UMTS_AMR   || 
       data->dType == UMTS_AMR_2
       )
		{
       if (dmacs > 0 && dmacs <= 8)
	       data->dmacs = dmacs;
       else
			{fprintf(stderr,"ERROR in input: dmacs not between 1..8\n");exit(0);}  //4.3.0 error text
		}

  if (dOM == 'y')
	  data->dOM = 1;
  else if (dOM == 'n')
	  data->dOM = 0;
  else
  {fprintf(stderr,"ERROR in input: dOM not 'y' or 'n'\n");exit(0);}  //4.3.0 error text

  /* check some settings */

  if (data->lType != FR_AMR && data->lType != HR_AMR &&
	  data->lType != UMTS_AMR && data->lType != UMTS_AMR_2)
    data->lOM = data->lscs = data->lacs =  data->lmacs = 0;

  if (data->dType != FR_AMR && data->dType != HR_AMR &&
	  data->dType != UMTS_AMR && data->dType != UMTS_AMR_2)
    data->dOM = data->dscs = data->dacs = data->dmacs = 0;

  if ((data->lacs & data->lscs) != data->lacs ||
      (data->dacs & data->dscs) != data->dacs)
    {fprintf(stderr,"ERROR: ACS not in SCS\n");exit(0);}

  if (data->lmacs < nModes(data->lacs) ||
      data->dmacs < nModes(data->dacs))
    {fprintf(stderr,"ERROR: MACS < #ACS\n");exit(0);}
  
  if (data->lType == HR_AMR)
    // was: {data->lscs &= 0x3f; data->lacs &= 0x3f;}       begin 4.3.0
	// this automatic removal of the highest modes was removed
	// instead an input error is given
	{ 
	if ( (data->lscs > 63) || (data->lacs > 63) )
		{
		fprintf(stderr,"Error in input: Highest two AMR mode not allowed for HR_AMR(local)\n");
		exit(0);
		}
	}													 // end 4.3.0

  if (data->dType == HR_AMR)
    // was: {data->dscs &= 0x3f; data->dacs &= 0x3f;}       begin 4.3.0
    // this automatic removal of the highest modes was removed
	// instead an input error is given
	{ 
	if ( (data->dscs > 63) || (data->dacs > 63) )
		{
		fprintf(stderr,"Error in input: Highest two AMR mode not allowed for HR_AMR(distant)\n");
		exit(0);
		}
	}													 // end 4.3.0

  return 1;  
}




/* print Data to stdout */
int printData(Data* data)
{
  CodecSet m;

  printf("\n\n------------------------------------------------------------------");
  printf("\n\n\t%s\t\t\t\t\t%s\n", codecName(data->lType), codecName(data->dType));
  printf("\tMACS = %d\t\t\t\tMACS = %d\n", data->lmacs, data->dmacs);
  printf("\tOM   = %s\t\t\t\tOM   = %s\n", data->lOM ? "yes" : "no", data->dOM ? "yes" : "no");
  
  /* Two compatible AMR codec types? */   // 4.3.0 condition for mode output introduced
  if (data->lType == UMTS_AMR_2 && data->dType == UMTS_AMR_2 ||
	  data->lType == UMTS_AMR   && data->dType == UMTS_AMR   ||
      data->lType == FR_AMR     && data->dType == FR_AMR     ||
      data->lType == HR_AMR     && data->dType == HR_AMR     ||
      data->lType == FR_AMR     && data->dType == HR_AMR     ||
      data->lType == HR_AMR     && data->dType == FR_AMR     ||
      data->lType == UMTS_AMR_2 && data->dType == HR_AMR     ||
      data->lType == HR_AMR     && data->dType == UMTS_AMR_2 ||
      data->lType == UMTS_AMR_2 && data->dType == FR_AMR     ||
      data->lType == FR_AMR     && data->dType == UMTS_AMR_2 ||
      data->lType == UMTS_AMR_2 && data->dType == UMTS_AMR   ||
      data->lType == UMTS_AMR   && data->dType == UMTS_AMR_2   )
	  {
	  printf("\n\tSCS\tACS\tIACS\tOACS\tCSCS\tACS\tSCS\n");

  
	  for (m=mode_122; m; m>>=1)
		{
		printf("%s\t%c\t%c\t%c\t%c\t%c\t%c\t%c\n",
				 modeName((CodecMode)m),
				 data->lscs & m ? 'X' : '-',
				 data->lacs & m ? 'X' : '-',
				 data->cacs & m ? 'X' : '-',
				 data->oacs & m ? 'X' : '-',
				 data->cscs & m ? 'X' : '-',
				 data->dacs & m ? 'X' : '-',
				 data->dscs & m ? 'X' : '-');
		}
	
	}
    
  printf("\n");
  return 0;
}





/* -------------------------------------------------------------------------------
                                      main
   ------------------------------------------------------------------------------- */

int main()
{
  Data a;
  Data* data = &a;
  

  
  while (getInputData(data))
  {
    determine_IACS(data);
    determine_OACS(data);
    printData(data);
    tfo_decision_main(data);
  }
  
    
  return 0;
}

